# -*- coding: utf-8 -*-
from collective.fingerpointing.testing import INTEGRATION_TESTING
from collective.fingerpointing.utils import get_request_information

import unittest


class UtilsTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer['request']

    def test_get_request_information(self):
        self.assertEqual(
            get_request_information(), ('test_user_1_', 'None'))

    def test_get_request_information_anonymous(self):
        from plone.app.testing import logout
        logout()
        self.assertEqual(get_request_information(), ('-', 'None'))

    def test_get_request_information_cloudflare(self):
        self.request.environ['HTTP_CF_CONNECTING_IP'] = '192.168.1.1'
        self.assertEqual(
            get_request_information(), ('test_user_1_', '192.168.1.1'))



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from plone.api.exc import InvalidParameterError
from zope.component import ComponentLookupError
from zope.lifecycleevent import IObjectAddedEvent
from zope.lifecycleevent import IObjectModifiedEvent
from zope.lifecycleevent import IObjectRemovedEvent


def lifecycle_logger(obj, event):
    """Log content type life cycle events like object creation,
    modification and removal.
    """
    # subscriber is registered even if package has not yet been installed
    # ignore any error caused by missing registry records
    try:
        record = IFingerPointingSettings.__identifier__ + '.audit_lifecycle'
        audit_lifecycle = api.portal.get_registry_record(record)
    except (ComponentLookupError, InvalidParameterError):
        return

    if audit_lifecycle:
        user, ip = get_request_information()

        if IObjectAddedEvent.providedBy(event):
            action = 'create'
            extras = u'object={0}'.format(repr(obj))
        if IObjectModifiedEvent.providedBy(event):
            action = 'modify'
            extras = u'object={0}'.format(repr(obj))
        if IObjectRemovedEvent.providedBy(event):
            action = 'remove'
            extras = u'object={0}'.format(repr(obj))

        log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
"""Setup testing infrastructure.

For Plone 5 we need to install plone.app.contenttypes.
"""
from plone import api
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import pkg_resources


try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    from plone.app.testing import PLONE_FIXTURE
else:
    from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE as PLONE_FIXTURE


IS_PLONE_5 = api.env.plone_version().startswith('5')


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    # XXX: this can probably be done in a different way
    def _configure_audit_log(self):
        """Fake configuration for collective.fingerpointing as tests
        expect an audit.log file to be set.
        """
        from collective.fingerpointing.config import fingerpointing_config
        fingerpointing_config['audit-log'] = '/tmp/audit.log'

    def setUpZope(self, app, configurationContext):
        self._configure_audit_log()
        import collective.fingerpointing
        self.loadZCML(package=collective.fingerpointing)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'collective.fingerpointing:default')
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')


FIXTURE = Fixture()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name='collective.fingerpointing:Integration')

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name='collective.fingerpointing:Functional')

ROBOT_TESTING = FunctionalTesting(
    bases=(FIXTURE, AUTOLOGIN_LIBRARY_FIXTURE, z2.ZSERVER_FIXTURE),
    name='collective.fingerpointing:Robot',
)



# -*- coding: utf-8 -*-
"""Event subscriber handler for IActionSucceededEvent."""
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from plone.api.exc import InvalidParameterError


def workflow_logger(event):
    """Log workflow transitions."""
    try:
        record = IFingerPointingSettings.__identifier__ + '.audit_workflow'
        audit_workflow = api.portal.get_registry_record(record)
    except InvalidParameterError:
        return  # package is not installed

    if audit_workflow:
        user, ip = get_request_information()
        action = 'workflow transition'
        extras = u'object={0} transition={1}'.format(event.object, event.action)
        log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
from plone import api
from zope.globalrequest import getRequest


def get_request_information():
    """Return logged in user name and remote IP address."""
    request = getRequest()
    try:
        user_id = api.user.get_current().getMemberId()
    except AttributeError:
        user_id = '-'  # anonymous user

    # honor Cloudflare real client IP address request header if present
    # see: https://support.cloudflare.com/hc/en-us/articles/200170986
    if 'HTTP_CF_CONNECTING_IP' in request.environ:
        ip = request.environ['HTTP_CF_CONNECTING_IP']
    # Common proxy configuration
    elif 'HTTP_X_FORWARDED_FOR' in request.environ:
        ip = request.environ['HTTP_X_FORWARDED_FOR']
    else:
        ip = request.getClientAddr() or 'None'  # return 'None' on tests

    return user_id, ip



# -*- coding: utf-8 -*-
from collective.fingerpointing.testing import INTEGRATION_TESTING
from plone import api
from plone.app.testing import logout

import unittest


class LogViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.view = api.content.get_view(
            'fingerpointing-audit-log', self.portal, self.request)

    def test_audit_log_view(self):
        view = self.view.__of__(self.portal)
        self.assertTrue(view())

    def test_audit_log_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse('@@fingerpointing-audit-log')

    def test_get_audit_log_files(self):
        audit_log_files = self.view._get_audit_log_files
        # we have at least one log file to process
        self.assertGreaterEqual(len(audit_log_files), 1)
        # the first file is the current audit log
        self.assertEqual('/tmp/audit.log', audit_log_files[0])

    def test_log_tail(self):
        from Products.PlonePAS.events import UserLoggedOutEvent
        from zope.event import notify
        event = UserLoggedOutEvent(self.request)
        notify(event)
        audit_log = self.view.get_audit_log.split('\n')
        # user logged out event is first on log (newer entries first)
        self.assertIn('user=test_user_1_ ip=None action=logout', audit_log[0])



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.iterate.interfaces import ICancelCheckoutEvent
from plone.app.iterate.interfaces import ICheckinEvent
from plone.app.iterate.interfaces import ICheckoutEvent


def iterate_logger(event):
    """Log events like content editing checkouts and checkins, if
    plone.app.iterate is installed.
    """
    # subscriber is registered even if package has not yet been installed
    # ignore any error caused by missing registry records
    try:
        record = IFingerPointingSettings.__identifier__ + '.audit_iterate'
        audit_iterate = api.portal.get_registry_record(record)
    except InvalidParameterError:
        return

    if audit_iterate:
        user, ip = get_request_information()

        if ICheckoutEvent.providedBy(event):
            action = 'checkout'
            extras = u'object={0} working_copy={1}'.format(
                event.object, event.working_copy)
        if ICheckinEvent.providedBy(event):
            action = 'checkin'
            extras = u'object={0} baseline={1}'.format(
                event.object, event.baseline)
        if ICancelCheckoutEvent.providedBy(event):
            action = 'cancel checkout'
            extras = u'object={0} baseline={1}'.format(
                event.object, event.baseline)

        log_info(AUDIT_MESSAGE.format(user, ip, action, extras))
